@@ -46,13 +46,15 @@
 
 cmakeBaseName = "cmake"
 cmakeDefaultVersion = "3.3.2"
-cmakeSupportedVersions = ["2.8.11", "3.1.1", "3.3.2", "3.4.0", "3.4.1"]
+cmakeSupportedVersions = ["2.8.11", "3.1.1", "3.3.2", "3.4.0", "3.4.1", "3.4.3", "3.5.1"]
 cmakeTarballVersions = {
   "2.8.11" : "2.8.11.2",
   "3.1.1" : "3.1.1",
   "3.3.2" : "3.3.2",
   "3.4.0" : "3.4.0",
-  "3.4.1" : "3.4.1"
+  "3.4.1" : "3.4.1",
+  "3.4.3" : "3.4.3",
+  "3.5.1" : "3.5.1",
   }
 
 # NOTES:
