@@ -49,9 +49,6 @@
 #include <Ifpack2_Experimental_RBILUK.hpp>
 #include <Ifpack2_Utilities.hpp>
 #include <Ifpack2_RILUK.hpp>
-// Need to include this if using the "classic" version of Tpetra,
-// since it may not get included in that case.
-#include <Kokkos_ArithTraits.hpp>
 
 namespace Ifpack2 {
 
@@ -79,6 +76,8 @@ template<class MatrixType>
 void
 RBILUK<MatrixType>::setMatrix (const Teuchos::RCP<const block_crs_matrix_type>& A)
 {
+  // FIXME (mfh 04 Nov 2015) What about A_?  When does that get (re)set?
+
   // It's legal for A to be null; in that case, you may not call
   // initialize() until calling setMatrix() with a nonnull input.
   // Regardless, setting the matrix invalidates any previous
@@ -181,15 +180,30 @@ void RBILUK<MatrixType>::initialize ()
   using Teuchos::RCP;
   using Teuchos::rcp;
   using Teuchos::rcp_dynamic_cast;
+  const char prefix[] = "Ifpack2::Experimental::RBILUK::initialize: ";
 
-  if (A_block_.is_null()) {
-    TEUCHOS_TEST_FOR_EXCEPTION(A_.is_null(), std::runtime_error, "Ifpack2::Experimental::RBILUK::initialize: "
-      "The matrix is null.  Please call setMatrix() with a nonnull input before calling this method.");
-    RCP<const LocalFilter<row_matrix_type> > filteredA = rcp_dynamic_cast<const LocalFilter<row_matrix_type> >(A_);
-    TEUCHOS_TEST_FOR_EXCEPTION(filteredA.is_null(), std::runtime_error, "Ifpack2::Experimental::RBILUK::initialize: "
-      "Cannot cast to filtered matrix.");
-    RCP<const OverlappingRowMatrix<row_matrix_type> > overlappedA = rcp_dynamic_cast<const OverlappingRowMatrix<row_matrix_type> >(filteredA->getUnderlyingMatrix());
-    if (overlappedA != Teuchos::null) {
+  // FIXME (mfh 04 Nov 2015) Apparently it's OK for A_ to be null.
+  // That probably means that this preconditioner was created with a
+  // BlockCrsMatrix directly, so it doesn't need the LocalFilter.
+
+  // TEUCHOS_TEST_FOR_EXCEPTION
+  //   (A_.is_null (), std::runtime_error, prefix << "The matrix (A_, the "
+  //    "RowMatrix) is null.  Please call setMatrix() with a nonnull input "
+  //    "before calling this method.");
+
+  if (A_block_.is_null ()) {
+    // FIXME (mfh 04 Nov 2015) Why does the input have to be a
+    // LocalFilter?  Why can't we just take a regular matrix, and
+    // apply a LocalFilter only if necessary, like other "local"
+    // Ifpack2 preconditioners already do?
+    RCP<const LocalFilter<row_matrix_type> > filteredA =
+      rcp_dynamic_cast<const LocalFilter<row_matrix_type> >(A_);
+    TEUCHOS_TEST_FOR_EXCEPTION
+      (filteredA.is_null (), std::runtime_error, prefix <<
+       "Cannot cast to filtered matrix.");
+    RCP<const OverlappingRowMatrix<row_matrix_type> > overlappedA =
+      rcp_dynamic_cast<const OverlappingRowMatrix<row_matrix_type> > (filteredA->getUnderlyingMatrix ());
+    if (! overlappedA.is_null ()) {
       A_block_ = rcp_dynamic_cast<const block_crs_matrix_type>(overlappedA->getUnderlyingMatrix());
     } else {
       //If there is no overlap, filteredA could be the block CRS matrix
@@ -197,10 +211,16 @@ void RBILUK<MatrixType>::initialize ()
     }
   }
 
-  TEUCHOS_TEST_FOR_EXCEPTION(
-    A_block_.is_null (), std::runtime_error, "Ifpack2::Experimental::RBILUK::initialize: "
-    "The matrix is null.  Please call setMatrix() with a nonnull input "
-    "before calling this method.");
+  TEUCHOS_TEST_FOR_EXCEPTION
+    (A_block_.is_null (), std::runtime_error, prefix << "The matrix (A_block_, "
+     "the BlockCrsMatrix) is null.  Please call setMatrix() with a nonnull "
+     "input before calling this method.");
+  TEUCHOS_TEST_FOR_EXCEPTION
+    (! A_block_->isFillComplete (), std::runtime_error, prefix << "The matrix "
+     "(A_block_, the BlockCrsMatrix) is not fill complete.  You may not invoke "
+     "initialize() or compute() with this matrix until the matrix is fill "
+     "complete.  Note: BlockCrsMatrix is fill complete if and only if its "
+     "underlying graph is fill complete.");
 
   blockSize_ = A_block_->getBlockSize();
 
@@ -244,7 +264,6 @@ void
 RBILUK<MatrixType>::
 initAllValues (const block_crs_matrix_type& A)
 {
-  //using Teuchos::ArrayRCP;
   using Teuchos::RCP;
   typedef Tpetra::Map<local_ordinal_type,global_ordinal_type,node_type> map_type;
 
@@ -287,6 +306,18 @@ initAllValues (const block_crs_matrix_type& A)
   U_block_->setAllToScalar (STM::zero ());
   D_block_->setAllToScalar (STM::zero ()); // Set diagonal values to zero
 
+  // NOTE (mfh 27 May 2016) The factorization below occurs entirely on
+  // host, so sync to host first.  The const_cast is unfortunate but
+  // is our only option to make this correct.
+  const_cast<block_crs_matrix_type&> (A).template sync<Kokkos::HostSpace> ();
+  L_block_->template sync<Kokkos::HostSpace> ();
+  U_block_->template sync<Kokkos::HostSpace> ();
+  D_block_->template sync<Kokkos::HostSpace> ();
+  // NOTE (mfh 27 May 2016) We're modifying L, U, and D on host.
+  L_block_->template modify<Kokkos::HostSpace> ();
+  U_block_->template modify<Kokkos::HostSpace> ();
+  D_block_->template modify<Kokkos::HostSpace> ();
+
   RCP<const map_type> rowMap = L_block_->getRowMap ();
 
   // First we copy the user's matrix into L and U, regardless of fill level.
@@ -368,28 +399,78 @@ initAllValues (const block_crs_matrix_type& A)
     }
   }
 
+  // NOTE (mfh 27 May 2016) Sync back to device, in case compute()
+  // ever gets a device implementation.
+  {
+    typedef typename block_crs_matrix_type::device_type device_type;
+    const_cast<block_crs_matrix_type&> (A).template sync<device_type> ();
+    L_block_->template sync<device_type> ();
+    U_block_->template sync<device_type> ();
+    D_block_->template sync<device_type> ();
+  }
   this->isInitialized_ = true;
 }
 
+namespace { // (anonymous)
+
+// For a given Kokkos::View type, possibly unmanaged, get the
+// corresponding managed Kokkos::View type.  This is handy for
+// translating from little_block_type or little_vec_type (both
+// possibly unmanaged) to their managed versions.
+template<class LittleBlockType>
+struct GetManagedView {
+  static_assert (Kokkos::Impl::is_view<LittleBlockType>::value,
+                 "LittleBlockType must be a Kokkos::View.");
+  typedef Kokkos::View<typename LittleBlockType::non_const_data_type,
+                       typename LittleBlockType::array_layout,
+                       typename LittleBlockType::device_type> managed_non_const_type;
+  static_assert (static_cast<int> (managed_non_const_type::rank) ==
+                 static_cast<int> (LittleBlockType::rank),
+                 "managed_non_const_type::rank != LittleBlock::rank.  "
+                 "Please report this bug to the Ifpack2 developers.");
+};
+
+} // namespace (anonymous)
+
 template<class MatrixType>
 void RBILUK<MatrixType>::compute ()
 {
+  typedef impl_scalar_type IST;
+  const char prefix[] = "Ifpack2::Experimental::RBILUK::compute: ";
+
   // initialize() checks this too, but it's easier for users if the
   // error shows them the name of the method that they actually
   // called, rather than the name of some internally called method.
-  TEUCHOS_TEST_FOR_EXCEPTION(
-    A_block_.is_null (), std::runtime_error, "Ifpack2::Experimental::RBILUK::compute: "
-    "The matrix is null.  Please call setMatrix() with a nonnull input "
-    "before calling this method.");
+  TEUCHOS_TEST_FOR_EXCEPTION
+    (A_block_.is_null (), std::runtime_error, prefix << "The matrix (A_block_, "
+     "the BlockCrsMatrix) is null.  Please call setMatrix() with a nonnull "
+     "input before calling this method.");
+  TEUCHOS_TEST_FOR_EXCEPTION
+    (! A_block_->isFillComplete (), std::runtime_error, prefix << "The matrix "
+     "(A_block_, the BlockCrsMatrix) is not fill complete.  You may not invoke "
+     "initialize() or compute() with this matrix until the matrix is fill "
+     "complete.  Note: BlockCrsMatrix is fill complete if and only if its "
+     "underlying graph is fill complete.");
 
   if (! this->isInitialized ()) {
     initialize (); // Don't count this in the compute() time
   }
 
-  typedef typename Tpetra::Details::GetLapackType<impl_scalar_type>::lapack_scalar_type LST;
-  typedef typename Tpetra::Details::GetLapackType<impl_scalar_type>::lapack_type lapack_type;
-
-  lapack_type lapack;
+  // NOTE (mfh 27 May 2016) The factorization below occurs entirely on
+  // host, so sync to host first.  The const_cast is unfortunate but
+  // is our only option to make this correct.
+  if (! A_block_.is_null ()) {
+    Teuchos::RCP<block_crs_matrix_type> A_nc =
+      Teuchos::rcp_const_cast<block_crs_matrix_type> (A_block_);
+    A_nc->template sync<Kokkos::HostSpace> ();
+  }
+  L_block_->template sync<Kokkos::HostSpace> ();
+  U_block_->template sync<Kokkos::HostSpace> ();
+  D_block_->template sync<Kokkos::HostSpace> ();
+  // NOTE (mfh 27 May 2016) We're modifying L, U, and D on host.
+  L_block_->template modify<Kokkos::HostSpace> ();
+  U_block_->template modify<Kokkos::HostSpace> ();
+  D_block_->template modify<Kokkos::HostSpace> ();
 
   Teuchos::Time timer ("RBILUK::compute");
   { // Start timing
@@ -411,21 +492,24 @@ void RBILUK<MatrixType>::compute ()
 
     const local_ordinal_type blockMatSize = blockSize_*blockSize_;
 
+    // FIXME (mfh 08 Nov 2015, 24 May 2016) We need to move away from
+    // expressing these strides explicitly, in order to finish #177
+    // (complete Kokkos-ization of BlockCrsMatrix) thoroughly.
     const local_ordinal_type rowStride = blockSize_;
-    const local_ordinal_type colStride = 1;
 
-    Teuchos::Array<int> ipiv(blockSize_);
-    Teuchos::Array<LST> work(1);
+    Teuchos::Array<int> ipiv_teuchos(blockSize_);
+    Kokkos::View<int*, Kokkos::HostSpace,
+      Kokkos::MemoryUnmanaged> ipiv (ipiv_teuchos.getRawPtr (), blockSize_);
+    Teuchos::Array<IST> work_teuchos(blockSize_);
+    Kokkos::View<IST*, Kokkos::HostSpace,
+      Kokkos::MemoryUnmanaged> work (work_teuchos.getRawPtr (), blockSize_);
 
     size_t num_cols = U_block_->getColMap()->getNodeNumElements();
     Teuchos::Array<int> colflag(num_cols);
 
-    Teuchos::Array<impl_scalar_type> diagMod(blockMatSize,STM::zero());
-    little_block_type diagModBlock(&diagMod[0], blockSize_, rowStride, colStride);
-    Teuchos::Array<impl_scalar_type> matTmpArray(blockMatSize,STM::zero());
-    little_block_type matTmp(&matTmpArray[0], blockSize_, rowStride, colStride);
-    Teuchos::Array<impl_scalar_type> multiplierArray(blockMatSize, STM::zero());
-    little_block_type multiplier(&multiplierArray[0], blockSize_, rowStride, colStride);
+    typename GetManagedView<little_block_type>::managed_non_const_type diagModBlock ("diagModBlock", blockSize_, blockSize_);
+    typename GetManagedView<little_block_type>::managed_non_const_type matTmp ("matTmp", blockSize_, blockSize_);
+    typename GetManagedView<little_block_type>::managed_non_const_type multiplier ("multiplier", blockSize_, blockSize_);
 
 //    Teuchos::ArrayRCP<scalar_type> DV = D_->get1dViewNonConst(); // Get view of diagonal
 
@@ -452,15 +536,17 @@ void RBILUK<MatrixType>::compute ()
       for (local_ordinal_type j = 0; j < NumL; ++j)
       {
         const local_ordinal_type matOffset = blockMatSize*j;
-        little_block_type lmat(&valsL[matOffset],blockSize_,rowStride, colStride);
-        little_block_type lmatV(&InV[matOffset],blockSize_,rowStride, colStride);
-        lmatV.assign(lmat);
+        little_block_type lmat((typename little_block_type::value_type*) &valsL[matOffset],blockSize_,rowStride);
+        little_block_type lmatV((typename little_block_type::value_type*) &InV[matOffset],blockSize_,rowStride);
+        //lmatV.assign(lmat);
+        Tpetra::Experimental::COPY (lmat, lmatV);
         InI[j] = colValsL[j];
       }
 
       little_block_type dmat = D_block_->getLocalBlock(local_row, local_row);
-      little_block_type dmatV(&InV[NumL*blockMatSize], blockSize_, rowStride, colStride);
-      dmatV.assign(dmat);
+      little_block_type dmatV((typename little_block_type::value_type*) &InV[NumL*blockMatSize], blockSize_, rowStride);
+      //dmatV.assign(dmat);
+      Tpetra::Experimental::COPY (dmat, dmatV);
       InI[NumL] = local_row;
 
       const local_ordinal_type * colValsU;
@@ -472,9 +558,10 @@ void RBILUK<MatrixType>::compute ()
         if (!(colValsU[j] < numLocalRows)) continue;
         InI[NumL+1+j] = colValsU[j];
         const local_ordinal_type matOffset = blockMatSize*(NumL+1+j);
-        little_block_type umat(&valsU[blockMatSize*j], blockSize_, rowStride, colStride);
-        little_block_type umatV(&InV[matOffset], blockSize_, rowStride, colStride);
-        umatV.assign(umat);
+        little_block_type umat((typename little_block_type::value_type*) &valsU[blockMatSize*j], blockSize_, rowStride);
+        little_block_type umatV((typename little_block_type::value_type*) &InV[matOffset], blockSize_, rowStride);
+        //umatV.assign(umat);
+        Tpetra::Experimental::COPY (umat, umatV);
         NumU += 1;
       }
       NumIn = NumL+NumU+1;
@@ -484,18 +571,22 @@ void RBILUK<MatrixType>::compute ()
         colflag[InI[j]] = j;
       }
 
-
       scalar_type diagmod = STM::zero (); // Off-diagonal accumulator
-      diagModBlock.fill(diagmod);
+      Kokkos::deep_copy (diagModBlock, diagmod);
 
       for (local_ordinal_type jj = 0; jj < NumL; ++jj) {
         local_ordinal_type j = InI[jj];
-        little_block_type currentVal(&InV[jj*blockMatSize], blockSize_, rowStride, colStride); // current_mults++;
-        multiplier.assign(currentVal);
+        little_block_type currentVal((typename little_block_type::value_type*) &InV[jj*blockMatSize], blockSize_, rowStride); // current_mults++;
+        //multiplier.assign(currentVal);
+        Tpetra::Experimental::COPY (currentVal, multiplier);
 
         const little_block_type dmatInverse = D_block_->getLocalBlock(j,j);
-        blockMatOpts.square_matrix_matrix_multiply(reinterpret_cast<impl_scalar_type*>(currentVal.getRawPtr()), reinterpret_cast<impl_scalar_type*>(dmatInverse.getRawPtr()), reinterpret_cast<impl_scalar_type*>(matTmp.getRawPtr()), blockSize_);
-        currentVal.assign(matTmp);
+        // alpha = 1, beta = 0
+        Tpetra::Experimental::GEMM ("N", "N", STS::one (), currentVal, dmatInverse,
+                                    STS::zero (), matTmp);
+        //blockMatOpts.square_matrix_matrix_multiply(reinterpret_cast<impl_scalar_type*> (currentVal.ptr_on_device ()), reinterpret_cast<impl_scalar_type*> (dmatInverse.ptr_on_device ()), reinterpret_cast<impl_scalar_type*> (matTmp.ptr_on_device ()), blockSize_);
+        //currentVal.assign(matTmp);
+        Tpetra::Experimental::COPY (matTmp, currentVal);
 
         const local_ordinal_type * UUI;
         scalar_type * UUV;
@@ -506,9 +597,11 @@ void RBILUK<MatrixType>::compute ()
             if (!(UUI[k] < numLocalRows)) continue;
             const int kk = colflag[UUI[k]];
             if (kk > -1) {
-              little_block_type kkval(&InV[kk*blockMatSize], blockSize_, rowStride, colStride);
-              little_block_type uumat(&UUV[k*blockMatSize], blockSize_, rowStride, colStride);
-              blockMatOpts.square_matrix_matrix_multiply(reinterpret_cast<impl_scalar_type*>(multiplier.getRawPtr()), reinterpret_cast<impl_scalar_type*>(uumat.getRawPtr()), reinterpret_cast<impl_scalar_type*>(kkval.getRawPtr()), blockSize_, -STM::one(), STM::one());
+              little_block_type kkval((typename little_block_type::value_type*) &InV[kk*blockMatSize], blockSize_, rowStride);
+              little_block_type uumat((typename little_block_type::value_type*) &UUV[k*blockMatSize], blockSize_, rowStride);
+              Tpetra::Experimental::GEMM ("N", "N", -STM::one (), multiplier, uumat,
+                                          STM::one (), kkval);
+              //blockMatOpts.square_matrix_matrix_multiply(reinterpret_cast<impl_scalar_type*> (multiplier.ptr_on_device ()), reinterpret_cast<impl_scalar_type*> (uumat.ptr_on_device ()), reinterpret_cast<impl_scalar_type*> (kkval.ptr_on_device ()), blockSize_, -STM::one(), STM::one());
             }
           }
         }
@@ -516,26 +609,32 @@ void RBILUK<MatrixType>::compute ()
           for (local_ordinal_type k = 0; k < NumUU; ++k) {
             if (!(UUI[k] < numLocalRows)) continue;
             const int kk = colflag[UUI[k]];
-            little_block_type uumat(&UUV[k*blockMatSize], blockSize_, rowStride, colStride);
+            little_block_type uumat((typename little_block_type::value_type*) &UUV[k*blockMatSize], blockSize_, rowStride);
             if (kk > -1) {
-              little_block_type kkval(&InV[kk*blockMatSize], blockSize_, rowStride, colStride);
-              blockMatOpts.square_matrix_matrix_multiply(reinterpret_cast<impl_scalar_type*>(multiplier.getRawPtr()), reinterpret_cast<impl_scalar_type*>(uumat.getRawPtr()), reinterpret_cast<impl_scalar_type*>(kkval.getRawPtr()), blockSize_, -STM::one(), STM::one());
+              little_block_type kkval((typename little_block_type::value_type*) &InV[kk*blockMatSize], blockSize_, rowStride);
+              Tpetra::Experimental::GEMM ("N", "N", -STM::one (), multiplier, uumat,
+                                          STM::one (), kkval);
+              //blockMatOpts.square_matrix_matrix_multiply(reinterpret_cast<impl_scalar_type*>(multiplier.ptr_on_device ()), reinterpret_cast<impl_scalar_type*>(uumat.ptr_on_device ()), reinterpret_cast<impl_scalar_type*>(kkval.ptr_on_device ()), blockSize_, -STM::one(), STM::one());
             }
             else {
-              blockMatOpts.square_matrix_matrix_multiply(reinterpret_cast<impl_scalar_type*>(multiplier.getRawPtr()), reinterpret_cast<impl_scalar_type*>(uumat.getRawPtr()), reinterpret_cast<impl_scalar_type*>(diagModBlock.getRawPtr()), blockSize_, -STM::one(), STM::one());
+              Tpetra::Experimental::GEMM ("N", "N", -STM::one (), multiplier, uumat,
+                                          STM::one (), diagModBlock);
+              //blockMatOpts.square_matrix_matrix_multiply(reinterpret_cast<impl_scalar_type*>(multiplier.ptr_on_device ()), reinterpret_cast<impl_scalar_type*>(uumat.ptr_on_device ()), reinterpret_cast<impl_scalar_type*>(diagModBlock.ptr_on_device ()), blockSize_, -STM::one(), STM::one());
             }
           }
         }
       }
       if (NumL) {
         // Replace current row of L
-        L_block_->replaceLocalValues(local_row, InI.getRawPtr(), InV.getRawPtr(), NumL);
+        L_block_->replaceLocalValues (local_row, InI.getRawPtr (), InV.getRawPtr (), NumL);
       }
 
-      dmat.assign(dmatV);
+      // dmat.assign(dmatV);
+      Tpetra::Experimental::COPY (dmatV, dmat);
 
       if (this->RelaxValue_ != STM::zero ()) {
-        dmat.update(this->RelaxValue_, diagModBlock);
+        //dmat.update(this->RelaxValue_, diagModBlock);
+        Tpetra::Experimental::AXPY (this->RelaxValue_, diagModBlock, dmat);
       }
 
 //      if (STS::magnitude (DV[i]) > STS::magnitude (MaxDiagonalValue)) {
@@ -548,39 +647,32 @@ void RBILUK<MatrixType>::compute ()
 //      }
 //      else
       {
-
-        LST* const d_raw = reinterpret_cast<LST*> (dmat.getRawPtr());
-        int lapackInfo;
+        int lapackInfo = 0;
         for (int k = 0; k < blockSize_; ++k) {
           ipiv[k] = 0;
         }
 
-        lapack.GETRF(blockSize_, blockSize_, d_raw, blockSize_, ipiv.getRawPtr(), &lapackInfo);
+        Tpetra::Experimental::GETF2 (dmat, ipiv, lapackInfo);
+        //lapack.GETRF(blockSize_, blockSize_, d_raw, blockSize_, ipiv.getRawPtr(), &lapackInfo);
         TEUCHOS_TEST_FOR_EXCEPTION(
           lapackInfo != 0, std::runtime_error, "Ifpack2::Experimental::RBILUK::compute: "
           "lapackInfo = " << lapackInfo << " which indicates an error in the factorization GETRF.");
 
-        int lwork = -1;
-        lapack.GETRI(blockSize_, d_raw, blockSize_, ipiv.getRawPtr(), work.getRawPtr(), lwork, &lapackInfo);
-        TEUCHOS_TEST_FOR_EXCEPTION(
-          lapackInfo != 0, std::runtime_error, "Ifpack2::Experimental::RBILUK::compute: "
-          "lapackInfo = " << lapackInfo << " which indicates an error in the matrix inverse GETRI.");
-
-        typedef typename Kokkos::Details::ArithTraits<impl_scalar_type>::mag_type ImplMagnitudeType;
-        ImplMagnitudeType worksize = Kokkos::Details::ArithTraits<impl_scalar_type>::magnitude(work[0]);
-        lwork = static_cast<int>(worksize);
-        work.resize(lwork);
-        lapack.GETRI(blockSize_, d_raw, blockSize_, ipiv.getRawPtr(), work.getRawPtr(), lwork, &lapackInfo);
+        Tpetra::Experimental::GETRI (dmat, ipiv, work, lapackInfo);
+        //lapack.GETRI(blockSize_, d_raw, blockSize_, ipiv.getRawPtr(), work.getRawPtr(), lwork, &lapackInfo);
         TEUCHOS_TEST_FOR_EXCEPTION(
           lapackInfo != 0, std::runtime_error, "Ifpack2::Experimental::RBILUK::compute: "
           "lapackInfo = " << lapackInfo << " which indicates an error in the matrix inverse GETRI.");
       }
 
       for (local_ordinal_type j = 0; j < NumU; ++j) {
-        little_block_type currentVal(&InV[(NumL+1+j)*blockMatSize], blockSize_, rowStride, colStride); // current_mults++;
+        little_block_type currentVal((typename little_block_type::value_type*) &InV[(NumL+1+j)*blockMatSize], blockSize_, rowStride); // current_mults++;
         // scale U by the diagonal inverse
-        blockMatOpts.square_matrix_matrix_multiply(reinterpret_cast<impl_scalar_type*>(dmat.getRawPtr()), reinterpret_cast<impl_scalar_type*>(currentVal.getRawPtr()), reinterpret_cast<impl_scalar_type*>(matTmp.getRawPtr()), blockSize_);
-        currentVal.assign(matTmp);
+        Tpetra::Experimental::GEMM ("N", "N", STS::one (), dmat, currentVal,
+                                    STS::zero (), matTmp);
+        //blockMatOpts.square_matrix_matrix_multiply(reinterpret_cast<impl_scalar_type*>(dmat.ptr_on_device ()), reinterpret_cast<impl_scalar_type*>(currentVal.ptr_on_device ()), reinterpret_cast<impl_scalar_type*>(matTmp.ptr_on_device ()), blockSize_);
+        //currentVal.assign(matTmp);
+        Tpetra::Experimental::COPY (matTmp, currentVal);
       }
 
       if (NumU) {
@@ -596,6 +688,19 @@ void RBILUK<MatrixType>::compute ()
 
   } // Stop timing
 
+  // Sync everything back to device, for efficient solves.
+  {
+    typedef typename block_crs_matrix_type::device_type device_type;
+    if (! A_block_.is_null ()) {
+      Teuchos::RCP<block_crs_matrix_type> A_nc =
+        Teuchos::rcp_const_cast<block_crs_matrix_type> (A_block_);
+      A_nc->template sync<device_type> ();
+    }
+    L_block_->template sync<device_type> ();
+    U_block_->template sync<device_type> ();
+    D_block_->template sync<device_type> ();
+  }
+
   this->isComputed_ = true;
   this->numCompute_ += 1;
   this->computeTime_ += timer.totalElapsedTime ();
@@ -639,13 +744,12 @@ apply (const Tpetra::MultiVector<scalar_type,local_ordinal_type,global_ordinal_t
   const local_ordinal_type blockMatSize = blockSize_*blockSize_;
 
   const local_ordinal_type rowStride = blockSize_;
-  const local_ordinal_type colStride = 1;
 
   BMV yBlock (Y, * (A_block_->getGraph ()->getDomainMap ()), blockSize_);
   const BMV xBlock (X, * (A_block_->getColMap ()), blockSize_);
 
   Teuchos::Array<scalar_type> lclarray(blockSize_);
-  little_vec_type lclvec(&lclarray[0], blockSize_, colStride);
+  little_vec_type lclvec((typename little_vec_type::value_type*)&lclarray[0], blockSize_);
   const scalar_type one = STM::one ();
   const scalar_type zero = STM::zero ();
 
@@ -670,7 +774,8 @@ apply (const Tpetra::MultiVector<scalar_type,local_ordinal_type,global_ordinal_t
             local_ordinal_type local_row = i;
             little_vec_type xval = xBlock.getLocalBlock(local_row,imv);
             little_vec_type cval = cBlock.getLocalBlock(local_row,imv);
-            cval.assign(xval);
+            //cval.assign(xval);
+            Tpetra::Experimental::COPY (xval, cval);
 
             local_ordinal_type NumL;
             const local_ordinal_type * colValsL;
@@ -684,9 +789,10 @@ apply (const Tpetra::MultiVector<scalar_type,local_ordinal_type,global_ordinal_t
               little_vec_type prevVal = cBlock.getLocalBlock(col, imv);
 
               const local_ordinal_type matOffset = blockMatSize*j;
-              little_block_type lij(&valsL[matOffset],blockSize_,rowStride, colStride);
+              little_block_type lij((typename little_block_type::value_type*) &valsL[matOffset],blockSize_,rowStride);
 
-              cval.matvecUpdate(-one, lij, prevVal);
+              //cval.matvecUpdate(-one, lij, prevVal);
+              Tpetra::Experimental::GEMV (-one, lij, prevVal, cval);
             }
           }
         }
@@ -703,7 +809,8 @@ apply (const Tpetra::MultiVector<scalar_type,local_ordinal_type,global_ordinal_t
             local_ordinal_type local_row = (numRows-1)-i;
             little_vec_type rval = rBlock.getLocalBlock(local_row,imv);
             little_vec_type yval = yBlock.getLocalBlock(local_row,imv);
-            yval.assign(rval);
+            //yval.assign(rval);
+            Tpetra::Experimental::COPY (rval, yval);
 
             local_ordinal_type NumU;
             const local_ordinal_type * colValsU;
@@ -717,9 +824,10 @@ apply (const Tpetra::MultiVector<scalar_type,local_ordinal_type,global_ordinal_t
               little_vec_type prevVal = yBlock.getLocalBlock(col, imv);
 
               const local_ordinal_type matOffset = blockMatSize*(NumU-1-j);
-              little_block_type uij(&valsU[matOffset], blockSize_, rowStride, colStride);
+              little_block_type uij((typename little_block_type::value_type*) &valsU[matOffset], blockSize_, rowStride);
 
-              yval.matvecUpdate(-one, uij, prevVal);
+              //yval.matvecUpdate(-one, uij, prevVal);
+              Tpetra::Experimental::GEMV (-one, uij, prevVal, yval);
             }
           }
         }
