@@ -52,6 +52,10 @@
 #include "Tpetra_Experimental_BlockCrsMatrix_decl.hpp"
 #include <type_traits>
 
+#ifdef HAVE_IFPACK2_AND_TPETRAKERNELS_EXPERIMENTAL
+#include <KokkosKernels_Handle.hpp>
+#endif
+
 namespace Teuchos {
   // forward declarations
   class ParameterList;
@@ -587,6 +591,21 @@ private:
                             global_ordinal_type, node_type> block_crs_matrix_type;
   typedef Tpetra::Experimental::BlockMultiVector<scalar_type, local_ordinal_type,
                             global_ordinal_type, node_type> block_multivector_type;
+
+#ifdef HAVE_IFPACK2_AND_TPETRAKERNELS_EXPERIMENTAL
+  typedef typename crs_matrix_type::local_matrix_type kokkos_csr_matrix;
+  typedef typename kokkos_csr_matrix::StaticCrsGraphType crs_graph_type;
+  typedef typename kokkos_csr_matrix::StaticCrsGraphType::row_map_type lno_row_view_t;
+  typedef typename kokkos_csr_matrix::StaticCrsGraphType::entries_type lno_nonzero_view_t;
+  typedef typename kokkos_csr_matrix::values_type scalar_nonzero_view_t;
+  typedef typename kokkos_csr_matrix::StaticCrsGraphType::device_type TemporaryWorkSpace;
+  typedef typename kokkos_csr_matrix::StaticCrsGraphType::device_type PersistentWorkSpace;
+  typedef typename kokkos_csr_matrix::StaticCrsGraphType::execution_space MyExecSpace;
+  typedef typename KokkosKernels::Experimental::KokkosKernelsHandle
+      <lno_row_view_t,lno_nonzero_view_t, scalar_nonzero_view_t,
+      MyExecSpace, TemporaryWorkSpace,PersistentWorkSpace > KernelHandle;
+  Teuchos::RCP<KernelHandle> kh;
+#endif
   //@}
   //! \name Unimplemented methods that you are syntactically forbidden to call.
   //@{
@@ -622,6 +641,12 @@ private:
         const Tpetra::MultiVector<scalar_type,local_ordinal_type,global_ordinal_type,node_type>& X,
               Tpetra::MultiVector<scalar_type,local_ordinal_type,global_ordinal_type,node_type>& Y) const;
 
+  //! Apply multi-threaded Gauss-Seidel to X, returning the result in Y.
+  void ApplyInverseMTGS_CrsMatrix(
+          const Tpetra::MultiVector<scalar_type,local_ordinal_type,global_ordinal_type,node_type>& X,
+                Tpetra::MultiVector<scalar_type,local_ordinal_type,global_ordinal_type,node_type>& Y) const;
+
+
   //! Apply Gauss-Seidel for a Tpetra::RowMatrix specialization.
   void ApplyInverseGS_RowMatrix(
         const Tpetra::MultiVector<scalar_type,local_ordinal_type,global_ordinal_type,node_type>& X,
@@ -644,6 +669,21 @@ private:
         const Tpetra::MultiVector<scalar_type,local_ordinal_type,global_ordinal_type,node_type>& X,
               Tpetra::MultiVector<scalar_type,local_ordinal_type,global_ordinal_type,node_type>& Y) const;
 
+  //! Apply symmetric multi-threaded Gauss-Seidel to X, returning the result in Y.
+  void ApplyInverseMTSGS_CrsMatrix(
+          const Tpetra::MultiVector<scalar_type,local_ordinal_type,global_ordinal_type,node_type>& X,
+                Tpetra::MultiVector<scalar_type,local_ordinal_type,global_ordinal_type,node_type>& Y) const;
+
+  void MTGaussSeidel (
+      const crs_matrix_type* crsMat,
+      Tpetra::MultiVector<scalar_type,local_ordinal_type,global_ordinal_type,node_type>& X,
+      const Tpetra::MultiVector<scalar_type,local_ordinal_type,global_ordinal_type,node_type>& B,
+      const Tpetra::MultiVector<scalar_type,local_ordinal_type,global_ordinal_type,node_type>& D,
+      const scalar_type& dampingFactor,
+      const Tpetra::ESweepDirection direction,
+      const int numSweeps,
+      const bool zeroInitialGuess) const;
+
   //! Apply symmetric Gauss-Seidel for a Tpetra::RowMatrix specialization.
   void ApplyInverseSGS_RowMatrix(
         const Tpetra::MultiVector<scalar_type,local_ordinal_type,global_ordinal_type,node_type>& X,
@@ -708,25 +748,6 @@ private:
   /// \endcode
   block_diag_type blockDiag_;
 
-  typedef Kokkos::View<int**, typename block_crs_matrix_type::device_type> pivots_type;
-  typedef Kokkos::View<int**, typename block_crs_matrix_type::device_type,
-                       Kokkos::MemoryUnmanaged> unmanaged_pivots_type;
-
-  /// \brief Pivots from LU factorization (with partial pivoting) of
-  ///   the BlockCrsMatrix's block diagonal.
-  ///
-  /// This is only allocated and used if the input matrix is a
-  /// Tpetra::BlockCrsMatrix.  In that case, Ifpack2::Relaxation does
-  /// block relaxation, using the (small dense) blocks in the
-  /// BlockCrsMatrix.
-  ///
-  /// To get the 1-D array of pivots corresponding to local (graph
-  /// a.k.a. "mesh") row index i, do the following:
-  /// \code
-  /// auto ipiv_i = Kokkos::subview (blockDiagFactPivots_, i, Kokkos::ALL ());
-  /// \endcode
-  pivots_type blockDiagFactPivots_;
-
   Teuchos::RCP<block_multivector_type> yBlockColumnPointMap_;
 
   //! How many times to apply the relaxation per apply() call.
@@ -794,7 +815,8 @@ private:
   /// These are only used if the matrix has a const ("static") graph.
   /// In that case, the offsets of the diagonal entries will never
   /// change, even if the values of the diagonal entries change.
-  Teuchos::ArrayRCP<size_t> diagOffsets_;
+  Kokkos::View<size_t*, typename node_type::device_type> diagOffsets_;
+
   /// \brief Whether we have precomputed offsets of diagonal entries.
   ///
   /// We need this flag because it is not enough just to test if
