@@ -45,6 +45,7 @@
 #include <iostream>
 #include <sstream>
 #include <stdexcept>
+#include <algorithm>
 #include <Kokkos_Macros.hpp>
 
 /* only compile this file if CUDA is enabled for Kokkos */
@@ -106,7 +107,7 @@ void DeepCopyAsyncCuda( void * dst , const void * src , size_t n) {
 
 namespace Kokkos {
 
-#if ! defined( KOKKOS_USING_EXPERIMENTAL_VIEW )
+#if ! KOKKOS_USING_EXP_VIEW
 
 namespace {
 
@@ -166,7 +167,7 @@ void CudaSpace::texture_object_attach(  Impl::AllocationTracker const & tracker
   texture_object_attach_impl( tracker, type_size, desc );
 }
 
-#endif /* #if ! defined( KOKKOS_USING_EXPERIMENTAL_VIEW ) */
+#endif /* #if ! KOKKOS_USING_EXP_VIEW */
 
 void CudaSpace::access_error()
 {
@@ -182,7 +183,7 @@ void CudaSpace::access_error( const void * const )
 
 /*--------------------------------------------------------------------------*/
 
-#if ! defined( KOKKOS_USING_EXPERIMENTAL_VIEW )
+#if ! KOKKOS_USING_EXP_VIEW
 
 Impl::AllocationTracker CudaUVMSpace::allocate_and_track( const std::string & label, const size_t size )
 {
@@ -197,7 +198,7 @@ void CudaUVMSpace::texture_object_attach(  Impl::AllocationTracker const & track
   texture_object_attach_impl( tracker, type_size, desc );
 }
 
-#endif /* #if ! defined( KOKKOS_USING_EXPERIMENTAL_VIEW ) */
+#endif /* #if ! KOKKOS_USING_EXP_VIEW */
 
 bool CudaUVMSpace::available()
 {
@@ -211,14 +212,14 @@ bool CudaUVMSpace::available()
 
 /*--------------------------------------------------------------------------*/
 
-#if ! defined( KOKKOS_USING_EXPERIMENTAL_VIEW )
+#if ! KOKKOS_USING_EXP_VIEW
 
 Impl::AllocationTracker CudaHostPinnedSpace::allocate_and_track( const std::string & label, const size_t size )
 {
   return Impl::AllocationTracker( allocator(), size, label);
 }
 
-#endif /* #if ! defined( KOKKOS_USING_EXPERIMENTAL_VIEW ) */
+#endif /* #if ! KOKKOS_USING_EXP_VIEW */
 
 } // namespace Kokkos
 
@@ -743,14 +744,25 @@ print_records( std::ostream & s , const Kokkos::CudaSpace & space , bool detail
         head.m_label[0] = 0 ;
       }
 
-      snprintf( buffer , 256 , "Cuda addr( 0x%.12lx ) list( 0x%.12lx 0x%.12lx ) extent[ 0x%.12lx + %.8ld ] count(%d) dealloc(0x%.12lx) %s\n"
-              , reinterpret_cast<unsigned long>( r )
-              , reinterpret_cast<unsigned long>( r->m_prev )
-              , reinterpret_cast<unsigned long>( r->m_next )
-              , reinterpret_cast<unsigned long>( r->m_alloc_ptr )
+      //Formatting dependent on sizeof(uintptr_t)
+      const char * format_string;
+
+      if (sizeof(uintptr_t) == sizeof(unsigned long)) { 
+        format_string = "Cuda addr( 0x%.12lx ) list( 0x%.12lx 0x%.12lx ) extent[ 0x%.12lx + %.8ld ] count(%d) dealloc(0x%.12lx) %s\n";
+      }
+      else if (sizeof(uintptr_t) == sizeof(unsigned long long)) { 
+        format_string = "Cuda addr( 0x%.12llx ) list( 0x%.12llx 0x%.12llx ) extent[ 0x%.12llx + %.8ld ] count(%d) dealloc(0x%.12llx) %s\n";
+      }
+
+      snprintf( buffer , 256 
+              , format_string
+              , reinterpret_cast<uintptr_t>( r )
+              , reinterpret_cast<uintptr_t>( r->m_prev )
+              , reinterpret_cast<uintptr_t>( r->m_next )
+              , reinterpret_cast<uintptr_t>( r->m_alloc_ptr )
               , r->m_alloc_size
               , r->m_count
-              , reinterpret_cast<unsigned long>( r->m_dealloc )
+              , reinterpret_cast<uintptr_t>( r->m_dealloc )
               , head.m_label
               );
       std::cout << buffer ;
@@ -763,8 +775,19 @@ print_records( std::ostream & s , const Kokkos::CudaSpace & space , bool detail
 
         Kokkos::Impl::DeepCopy<HostSpace,CudaSpace>::DeepCopy( & head , r->m_alloc_ptr , sizeof(SharedAllocationHeader) );
 
-        snprintf( buffer , 256 , "Cuda [ 0x%.12lx + %ld ] %s\n"
-                , reinterpret_cast< unsigned long >( r->data() )
+        //Formatting dependent on sizeof(uintptr_t)
+        const char * format_string;
+
+        if (sizeof(uintptr_t) == sizeof(unsigned long)) { 
+          format_string = "Cuda [ 0x%.12lx + %ld ] %s\n";
+        }
+        else if (sizeof(uintptr_t) == sizeof(unsigned long long)) { 
+          format_string = "Cuda [ 0x%.12llx + %ld ] %s\n";
+        }
+
+        snprintf( buffer , 256 
+                , format_string
+                , reinterpret_cast< uintptr_t >( r->data() )
                 , r->size()
                 , head.m_label
                 );
