@@ -48,11 +48,166 @@
 #include <impl/Kokkos_Traits.hpp>
 #include <impl/Kokkos_StaticAssert.hpp>
 #include <impl/Kokkos_Tags.hpp>
-
+#include <iostream>
 //----------------------------------------------------------------------------
 
 namespace Kokkos {
 
+//Schedules for Execution Policies
+struct Static {
+};
+
+struct Dynamic {
+};
+
+//Schedule Wrapper Type
+template<class ScheduleType>
+struct Schedule {
+  static_assert(std::is_same<ScheduleType,Static>::value ||
+                std::is_same<ScheduleType,Dynamic>::value,
+                "Kokkos: Invalid Schedule<> type.");
+  typedef Schedule<ScheduleType> schedule_type;
+  typedef ScheduleType type;
+};
+
+//Specif Iteration Index Type
+template<typename iType>
+struct IndexType {
+  static_assert(std::is_integral<iType>::value,"Kokkos: Invalid IndexType<>.");
+  typedef IndexType<iType> index_type;
+  typedef iType type;
+};
+
+namespace Impl {
+
+template<class Arg>
+struct is_schedule_type {
+  enum { value = 0};
+};
+
+template<class ScheduleType>
+struct is_schedule_type<Schedule<ScheduleType> > {
+  enum {value = 1 };
+};
+
+template<class Arg>
+struct is_index_type {
+  enum { value = 0 };
+};
+
+template<typename iType>
+struct is_index_type<IndexType<iType> > {
+  enum { value = 1 };
+};
+
+template<typename Arg>
+struct is_tag_type {
+  enum { value = !(is_execution_space<Arg>::value ||
+                   is_schedule_type<Arg>::value ||
+                   is_index_type<Arg>::value ||
+                   std::is_integral<Arg>::value)};
+};
+
+//Policy Traits
+template<class ... Properties>
+struct PolicyTraits;
+
+template<>
+struct PolicyTraits<void> {
+  typedef void execution_space;
+  typedef void schedule_type;
+  typedef void index_type;
+  typedef void tag_type;
+};
+
+
+//Strip off ExecutionSpace
+template<class ExecutionSpace, class ... Props>
+struct PolicyTraits<typename std::enable_if<is_execution_space<ExecutionSpace>::value >::type,ExecutionSpace,Props ...> {
+  static_assert( std::is_same<typename PolicyTraits<void, Props ...>::execution_space, void>::value,
+                 "ExecutionPolicy: Only one execution space template argument may be used.");
+  typedef ExecutionSpace execution_space;
+  typedef typename PolicyTraits<void, Props ...>::schedule_type schedule_type;
+  typedef typename PolicyTraits<void, Props ...>::index_type index_type;
+  typedef typename PolicyTraits<void, Props ...>::tag_type tag_type;
+};
+
+//Strip off ScheduleType
+template<class ScheduleType, class ... Props>
+struct PolicyTraits<typename std::enable_if<is_schedule_type<Schedule<ScheduleType> >::value >::type,Schedule<ScheduleType>,Props ...> {
+  static_assert( std::is_same<typename PolicyTraits<void, Props ...>::schedule_type, void>::value,
+                 "ExecutionPolicy: Only one Schedule<..> template argument may be used.");
+  typedef typename PolicyTraits<void, Props ...>::execution_space execution_space;
+  typedef ScheduleType schedule_type;
+  typedef typename PolicyTraits<void, Props ...>::index_type index_type;
+  typedef typename PolicyTraits<void, Props ...>::tag_type tag_type;
+};
+
+//Strip off IndexType
+template<typename iType, class ... Props>
+struct PolicyTraits<void, IndexType<iType>,Props ...> {
+  static_assert( std::is_same<typename PolicyTraits<void, Props ...>::index_type, void>::value,
+                 "ExecutionPolicy: Only one IndexType<..> template argument may be used.");
+  typedef typename PolicyTraits<void, Props ...>::execution_space execution_space;
+  typedef typename PolicyTraits<void, Props ...>::schedule_type schedule_type;
+  typedef iType index_type;
+  typedef typename PolicyTraits<void, Props ...>::tag_type tag_type;
+};
+
+//Strip off raw IndexType
+template<typename iType, class ... Props>
+struct PolicyTraits<typename std::enable_if<std::is_integral<iType>::value>::type, iType,Props ...> {
+  static_assert( std::is_same<typename PolicyTraits<void, Props ...>::index_type, void>::value,
+                 "ExecutionPolicy: Only one IndexType<..> template argument may be used.");
+  typedef typename PolicyTraits<void, Props ...>::execution_space execution_space;
+  typedef typename PolicyTraits<void, Props ...>::schedule_type schedule_type;
+  typedef iType index_type;
+  typedef typename PolicyTraits<void, Props ...>::tag_type tag_type;
+};
+
+//Strip off TagType
+template<class TagType, class ... Props>
+struct PolicyTraits<typename std::enable_if<!is_schedule_type<TagType>::value &&
+                                            !is_execution_space<TagType>::value &&
+                                            !is_index_type<TagType>::value &&
+                                            !std::is_integral<TagType>::value 
+                                           >::type,
+                    TagType,Props ...> {
+  static_assert( std::is_same<typename PolicyTraits<void, Props ...>::tag_type, void>::value,
+                 "ExecutionPolicy: Only one tag type template argument may be used.");
+
+  typedef typename PolicyTraits<void, Props ...>::execution_space execution_space;
+  typedef typename PolicyTraits<void, Props ...>::schedule_type schedule_type;
+  typedef typename PolicyTraits<void, Props ...>::index_type index_type;
+  typedef TagType tag_type;
+};
+
+
+template<class ... Props>
+struct PolicyTraits {
+#ifdef KOKKOS_DIRECT_VARIADIC_EXPANSION
+  typedef typename std::conditional<std::is_same<void, typename PolicyTraits<void, Props ...>::execution_space>::value,
+    Kokkos::DefaultExecutionSpace, typename PolicyTraits<void,Props ...>::execution_space>::type execution_space;
+  typedef typename std::conditional<std::is_same<void, typename PolicyTraits<void, Props ...>::schedule_type>::value,
+    Kokkos::Static, typename PolicyTraits<void,Props ...>::schedule_type>::type schedule_type;
+  typedef typename std::conditional<std::is_same<void, typename PolicyTraits<void, Props ...>::index_type>::value,
+    typename execution_space::size_type, typename PolicyTraits<void,Props ...>::index_type>::type index_type;
+  typedef typename std::conditional<std::is_same<void, typename PolicyTraits<void, Props ...>::tag_type>::value, 
+    void, typename PolicyTraits<void,Props ...>::tag_type>::type work_tag;
+#else
+  typedef typename has_condition<Kokkos::DefaultExecutionSpace,is_execution_space,Props ...>::type execution_space;
+  typedef typename has_condition<Kokkos::Schedule<Kokkos::Static>,is_schedule_type,Props ...>::type schedule_type;
+  typedef typename has_condition<void,is_tag_type,Props ...>::type work_tag;
+  typedef typename has_condition<typename execution_space::size_type, std::is_integral, Props ... >::type default_index_type;
+  typedef typename has_condition<Kokkos::IndexType<default_index_type>,is_index_type,Props ...>::type::type index_type;
+#endif
+};
+
+}
+
+}
+
+namespace Kokkos {
 /** \brief  Execution policy for work over a range of an integral type.
  *
  * Valid template argument options:
@@ -74,116 +229,102 @@ namespace Kokkos {
  *
  *  Blocking is the granularity of partitioning the range among threads.
  */
-template< class Arg0 = void , class Arg1 = void , class Arg2 = void 
-        , class ExecSpace =
-          // The first argument is the execution space,
-          // otherwise use the default execution space.
-          typename std::conditional
-            < Impl::is_execution_space< Arg0 >::value , Arg0
-            , Kokkos::DefaultExecutionSpace >::type
-        >
-class RangePolicy {
+template<class ... Properties>
+class RangePolicy: public Impl::PolicyTraits<Properties ... > {
 private:
 
-  // Default integral type and blocking factor:
-  typedef int DefaultIntType ;
-  enum { DefaultIntValue = 8 };
-
-  enum { Arg0_Void = Impl::is_same< Arg0 , void >::value };
-  enum { Arg1_Void = Impl::is_same< Arg1 , void >::value };
-  enum { Arg2_Void = Impl::is_same< Arg2 , void >::value };
-
-  enum { Arg0_ExecSpace = Impl::is_execution_space< Arg0 >::value };
-
-  enum { Arg0_IntConst = Impl::is_integral_constant< Arg0 >::value };
-  enum { Arg1_IntConst = Impl::is_integral_constant< Arg1 >::value };
-  enum { Arg2_IntConst = Impl::is_integral_constant< Arg2 >::value };
-
-  enum { Arg0_IntType = Impl::is_integral< Arg0 >::value };
-  enum { Arg1_IntType = Impl::is_integral< Arg1 >::value };
-  enum { Arg2_IntType = Impl::is_integral< Arg2 >::value };
-
-  enum { Arg0_WorkTag = ! Arg0_ExecSpace && ! Arg0_IntConst && ! Arg0_IntType && ! Arg0_Void };
-  enum { Arg1_WorkTag =   Arg0_ExecSpace && ! Arg1_IntConst && ! Arg1_IntType && ! Arg1_Void };
-
-  enum { ArgOption_OK = Impl::StaticAssert< (
-    ( Arg0_ExecSpace && Arg1_WorkTag && ( Arg2_IntConst || Arg2_IntType ) ) ||
-    ( Arg0_ExecSpace && Arg1_WorkTag && Arg2_Void ) ||
-    ( Arg0_ExecSpace && ( Arg1_IntConst || Arg1_IntType ) && Arg2_Void ) ||
-    ( Arg0_ExecSpace && Arg1_Void && Arg2_Void ) ||
-    ( Arg0_WorkTag && ( Arg1_IntConst || Arg1_IntType ) && Arg2_Void ) ||
-    ( Arg0_WorkTag && Arg1_Void && Arg2_Void ) ||
-    ( ( Arg0_IntConst || Arg0_IntType ) && Arg1_Void && Arg2_Void ) ||
-    ( Arg0_Void && Arg1_Void && Arg2_Void )
-    ) >::value };
-
-  // The work argument tag is the first or second argument
-  typedef typename std::conditional< Arg0_WorkTag , Arg0 ,
-          typename std::conditional< Arg1_WorkTag , Arg1 , void
-          >::type >::type
-    WorkTag ;
-
-  enum { Granularity = Arg0_IntConst ? unsigned(Impl::is_integral_constant<Arg0>::integral_value) : (
-                       Arg1_IntConst ? unsigned(Impl::is_integral_constant<Arg1>::integral_value) : (
-                       Arg2_IntConst ? unsigned(Impl::is_integral_constant<Arg2>::integral_value) : (
-                                       unsigned(DefaultIntValue) ))) };
-
-  // Only accept the integral type if the blocking is a power of two
-  static_assert( Impl::is_integral_power_of_two( Granularity )
-               , "RangePolicy blocking granularity must be power of two" );
-
-  typedef typename std::conditional< Arg0_IntType , Arg0 ,
-          typename std::conditional< Arg1_IntType , Arg1 ,
-          typename std::conditional< Arg2_IntType , Arg2 ,
-          typename std::conditional< Arg0_IntConst , typename Impl::is_integral_constant<Arg0>::integral_type ,
-          typename std::conditional< Arg1_IntConst , typename Impl::is_integral_constant<Arg1>::integral_type ,
-          typename std::conditional< Arg2_IntConst , typename Impl::is_integral_constant<Arg2>::integral_type ,
-                                                     DefaultIntType
-          >::type >::type >::type
-          >::type >::type >::type
-    IntType ;
-
-  enum { GranularityMask = IntType(Granularity) - 1 };
-
-  ExecSpace m_space ;
-  IntType   m_begin ;
-  IntType   m_end ;
+  typedef Impl::PolicyTraits<Properties ... > traits;
 
+  typename traits::execution_space m_space ;
+  typename traits::index_type  m_begin ;
+  typename traits::index_type  m_end ;
+  typename traits::index_type  m_granularity ;
+  typename traits::index_type  m_granularity_mask ;
 public:
 
   //! Tag this class as an execution policy
-  typedef ExecSpace    execution_space ;
-  typedef RangePolicy  execution_policy ;
-  typedef WorkTag      work_tag ;
-  typedef IntType      member_type ;
+  typedef typename traits::index_type member_type ;
 
-  KOKKOS_INLINE_FUNCTION const execution_space & space() const { return m_space ; }
+  KOKKOS_INLINE_FUNCTION const typename traits::execution_space & space() const { return m_space ; }
   KOKKOS_INLINE_FUNCTION member_type begin() const { return m_begin ; }
   KOKKOS_INLINE_FUNCTION member_type end()   const { return m_end ; }
 
+
+  //TODO: find a better workaround for Clangs weird instantiation order
+  // This thing is here because of an instantiation error, where the RangePolicy is inserted into FunctorValue Traits, which
+  // tries decltype on the operator. It tries to do this even though the first argument of parallel for clearly doesn't match.
+  void operator()(const int&) const {}
+
+  RangePolicy(const RangePolicy&) = default;
+  RangePolicy(RangePolicy&&) = default;
+
   inline RangePolicy() : m_space(), m_begin(0), m_end(0) {}
 
   /** \brief  Total range */
   inline
-  RangePolicy( const member_type work_begin
-             , const member_type work_end
-             )
-    : m_space()
-    , m_begin( work_begin < work_end ? work_begin : 0 )
-    , m_end(   work_begin < work_end ? work_end : 0 )
-    {}
-
-  /** \brief  Total range */
-  inline
-  RangePolicy( const execution_space & work_space
+  RangePolicy( const typename traits::execution_space & work_space
              , const member_type work_begin
              , const member_type work_end
              )
     : m_space( work_space )
     , m_begin( work_begin < work_end ? work_begin : 0 )
     , m_end(   work_begin < work_end ? work_end : 0 )
+    , m_granularity(0)
+    , m_granularity_mask(0)
+    {
+      set_auto_chunk_size();
+    }
+
+  /** \brief  Total range */
+  inline
+  RangePolicy( const member_type work_begin
+             , const member_type work_end
+             )
+    : RangePolicy( typename traits::execution_space()
+                 , work_begin , work_end )
     {}
 
+  public:
+
+     /** \brief return chunk_size */
+     inline member_type chunk_size() const {
+       return m_granularity;
+     }
+
+     /** \brief set chunk_size to a discrete value*/
+     inline RangePolicy set_chunk_size(int chunk_size_) const {
+       RangePolicy p = *this;
+       p.m_granularity = chunk_size_;
+       p.m_granularity_mask = p.m_granularity - 1;
+       return p;
+     }
+
+  private:
+     /** \brief finalize chunk_size if it was set to AUTO*/
+     inline void set_auto_chunk_size() {
+
+       typename traits::index_type concurrency = traits::execution_space::concurrency();
+       if( concurrency==0 ) concurrency=1;
+
+       if(m_granularity > 0) {
+         if(!Impl::is_integral_power_of_two( m_granularity ))
+           Kokkos::abort("RangePolicy blocking granularity must be power of two" );
+       }
+
+
+       member_type new_chunk_size = 1;
+       while(new_chunk_size*100*concurrency < m_end-m_begin)
+         new_chunk_size *= 2;
+       if(new_chunk_size < 128) {
+         new_chunk_size = 1;
+         while( (new_chunk_size*40*concurrency < m_end-m_begin ) && (new_chunk_size<128) )
+           new_chunk_size*=2;
+       }
+       m_granularity = new_chunk_size;
+       m_granularity_mask = m_granularity - 1;
+     }
+
+  public:
   /** \brief  Subrange for a partition's rank and size.
    *
    *  Typically used to partition a range over a group of threads.
@@ -211,7 +352,7 @@ public:
           // Split evenly among partitions, then round up to the granularity.
           const member_type work_part =
             ( ( ( ( range.end() - range.begin() ) + ( part_size - 1 ) ) / part_size )
-              + GranularityMask ) & ~member_type(GranularityMask);
+              + range.m_granularity_mask ) & ~member_type(range.m_granularity_mask);
 
           m_begin = range.begin() + work_part * part_rank ;
           m_end   = m_begin       + work_part ;
@@ -225,6 +366,7 @@ public:
      member_type m_end ;
      WorkRange();
      WorkRange & operator = ( const WorkRange & );
+   
   };
 };
 
@@ -267,51 +409,16 @@ public:
 
 }
 
-/** \brief  Execution policy for parallel work over a league of teams of threads.
- *
- *  The work functor is called for each thread of each team such that
- *  the team's member threads are guaranteed to be concurrent.
- *
- *  The team's threads have access to team shared scratch memory and
- *  team collective operations.
- *
- *  If the WorkTag is non-void then the first calling argument of the
- *  work functor's parentheses operator is 'const WorkTag &'.
- *  This allows a functor to have multiple work member functions.
- *
- *  template argument option with specified execution space:
- *    < ExecSpace , WorkTag >
- *    < ExecSpace , void >
- *
- *  template argument option with default execution space:
- *    < WorkTag , void >
- *    < void , void >
- */
-template< class Arg0 = void
-        , class Arg1 = void
-        , class ExecSpace =
-          // If the first argument is not an execution
-          // then use the default execution space.
-          typename std::conditional
-            < Impl::is_execution_space< Arg0 >::value , Arg0
-            , Kokkos::DefaultExecutionSpace >::type
-        >
-class TeamPolicy {
+namespace Impl {
+
+
+template< class ExecSpace, class ... Properties>
+class TeamPolicyInternal: public Impl::PolicyTraits<Properties ... > {
 private:
-
-  enum { Arg0_ExecSpace = Impl::is_execution_space< Arg0 >::value };
-  enum { Arg1_Void      = Impl::is_same< Arg1 , void >::value };
-  enum { ArgOption_OK   = Impl::StaticAssert< ( Arg0_ExecSpace || Arg1_Void ) >::value };
-
-  typedef typename std::conditional< Arg0_ExecSpace , Arg1 , Arg0 >::type WorkTag ;
+  typedef Impl::PolicyTraits<Properties ... > traits;
 
 public:
 
-  //! Tag this class as an execution policy
-  typedef TeamPolicy  execution_policy ;
-  typedef ExecSpace   execution_space ;
-  typedef WorkTag     work_tag ;
-
   //----------------------------------------
   /** \brief  Query maximum team size for a given functor.
    *
@@ -335,20 +442,20 @@ public:
   static int team_size_recommended( const FunctorType & , const int&);
   //----------------------------------------
   /** \brief  Construct policy with the given instance of the execution space */
-  TeamPolicy( const execution_space & , int league_size_request , int team_size_request , int vector_length_request = 1 );
+  TeamPolicyInternal( const typename traits::execution_space & , int league_size_request , int team_size_request , int vector_length_request = 1 );
 
-  TeamPolicy( const execution_space & , int league_size_request , const Kokkos::AUTO_t & , int vector_length_request = 1 );
+  TeamPolicyInternal( const typename traits::execution_space & , int league_size_request , const Kokkos::AUTO_t & , int vector_length_request = 1 );
 
   /** \brief  Construct policy with the default instance of the execution space */
-  TeamPolicy( int league_size_request , int team_size_request , int vector_length_request = 1 );
+  TeamPolicyInternal( int league_size_request , int team_size_request , int vector_length_request = 1 );
 
-  TeamPolicy( int league_size_request , const Kokkos::AUTO_t & , int vector_length_request = 1 );
+  TeamPolicyInternal( int league_size_request , const Kokkos::AUTO_t & , int vector_length_request = 1 );
 
   template<class MemorySpace>
-  TeamPolicy( int league_size_request , int team_size_request , const Experimental::TeamScratchRequest<MemorySpace>& team_scratch_memory_request );
+  TeamPolicyInternal( int league_size_request , int team_size_request , const Experimental::TeamScratchRequest<MemorySpace>& team_scratch_memory_request );
 
   template<class MemorySpace>
-  TeamPolicy( int league_size_request , const Kokkos::AUTO_t & , const Experimental::TeamScratchRequest<MemorySpace>& team_scratch_memory_request );
+  TeamPolicyInternal( int league_size_request , const Kokkos::AUTO_t & , const Experimental::TeamScratchRequest<MemorySpace>& team_scratch_memory_request );
 
   /** \brief  The actual league size (number of teams) of the policy.
    *
@@ -364,6 +471,10 @@ public:
    */
   KOKKOS_INLINE_FUNCTION int team_size() const ;
 
+  inline typename traits::index_type chunk_size() const ;
+
+  inline TeamPolicyInternal set_chunk_size(int chunk_size) const ;
+
   /** \brief  Parallel execution of a functor calls the functor once with
    *          each member of the execution policy.
    */
@@ -371,7 +482,7 @@ public:
 
     /** \brief  Handle to the currently executing team shared scratch memory */
     KOKKOS_INLINE_FUNCTION
-    typename execution_space::scratch_memory_space team_shmem() const ;
+    typename traits::execution_space::scratch_memory_space team_shmem() const ;
 
     /** \brief  Rank of this team within the league of teams */
     KOKKOS_INLINE_FUNCTION int league_rank() const ;
@@ -415,6 +526,104 @@ public:
     KOKKOS_INLINE_FUNCTION Type team_scan( const Type & value , Type * const global_accum ) const ;
   };
 };
+}
+
+namespace Impl {
+  struct PerTeamValue {
+    int value;
+    PerTeamValue(int arg);
+  };
+
+  struct PerThreadValue {
+    int value;
+    PerThreadValue(int arg);
+  };
+}
+
+Impl::PerTeamValue PerTeam(const int& arg);
+Impl::PerThreadValue PerThread(const int& arg);
+
+
+/** \brief  Execution policy for parallel work over a league of teams of threads.
+ *
+ *  The work functor is called for each thread of each team such that
+ *  the team's member threads are guaranteed to be concurrent.
+ *
+ *  The team's threads have access to team shared scratch memory and
+ *  team collective operations.
+ *
+ *  If the WorkTag is non-void then the first calling argument of the
+ *  work functor's parentheses operator is 'const WorkTag &'.
+ *  This allows a functor to have multiple work member functions.
+ *
+ *  Order of template arguments does not matter, since the implementation
+ *  uses variadic templates. Each and any of the template arguments can
+ *  be omitted.
+ *
+ *  Possible Template arguments and there default values:
+ *    ExecutionSpace (DefaultExecutionSpace): where to execute code. Must be enabled.
+ *    WorkTag (none): Tag which is used as the first argument for the functor operator.
+ *    Schedule<Type> (Schedule<Static>): Scheduling Policy (Dynamic, or Static).
+ *    IndexType<Type> (IndexType<ExecutionSpace::size_type>: Integer Index type used to iterate over the Index space.
+ */
+template< class ... Properties>
+class TeamPolicy: public
+  Impl::TeamPolicyInternal<
+     typename Impl::PolicyTraits<Properties ... >::execution_space,
+     Properties ...> {
+  typedef Impl::TeamPolicyInternal<
+       typename Impl::PolicyTraits<Properties ... >::execution_space,
+       Properties ...> internal_policy;
+  typedef Impl::PolicyTraits<Properties ... > traits;
+
+public:
+
+  TeamPolicy& operator = (const TeamPolicy&) = default;
+ 
+  /** \brief  Construct policy with the given instance of the execution space */
+  TeamPolicy( const typename traits::execution_space & , int league_size_request , int team_size_request , int vector_length_request = 1 )
+    : internal_policy(typename traits::execution_space(),league_size_request,team_size_request, vector_length_request) {}
+
+  TeamPolicy( const typename traits::execution_space & , int league_size_request , const Kokkos::AUTO_t & , int vector_length_request = 1 )
+    : internal_policy(typename traits::execution_space(),league_size_request,Kokkos::AUTO(), vector_length_request) {}
+
+  /** \brief  Construct policy with the default instance of the execution space */
+  TeamPolicy( int league_size_request , int team_size_request , int vector_length_request = 1 )
+    : internal_policy(league_size_request,team_size_request, vector_length_request) {}
+
+  TeamPolicy( int league_size_request , const Kokkos::AUTO_t & , int vector_length_request = 1 )
+    : internal_policy(league_size_request,Kokkos::AUTO(), vector_length_request) {}
+
+  template<class MemorySpace>
+  TeamPolicy( int league_size_request , int team_size_request , const Experimental::TeamScratchRequest<MemorySpace>& team_scratch_memory_request )
+    : internal_policy(league_size_request,team_size_request, team_scratch_memory_request) {}
+
+  template<class MemorySpace>
+  TeamPolicy( int league_size_request , const Kokkos::AUTO_t & , const Experimental::TeamScratchRequest<MemorySpace>& team_scratch_memory_request )
+    : internal_policy(league_size_request,Kokkos::AUTO(), team_scratch_memory_request) {}
+
+private:
+  TeamPolicy(const internal_policy& p):internal_policy(p) {}
+public:
+
+  inline TeamPolicy set_chunk_size(int chunk) const {
+    return TeamPolicy(internal_policy::set_chunk_size(chunk));
+  };
+
+  inline TeamPolicy set_scratch_size(const int& level, const Impl::PerTeamValue& per_team) const {
+    return TeamPolicy(internal_policy::set_scratch_size(level,per_team));
+  };
+  inline TeamPolicy set_scratch_size(const int& level, const Impl::PerThreadValue& per_thread) const {
+    return TeamPolicy(internal_policy::set_scratch_size(level,per_thread));
+  };
+  inline TeamPolicy set_scratch_size(const int& level, const Impl::PerTeamValue& per_team, const Impl::PerThreadValue& per_thread) const {
+    return TeamPolicy(internal_policy::set_scratch_size(level, per_team, per_thread));
+  };
+  inline TeamPolicy set_scratch_size(const int& level, const Impl::PerThreadValue& per_thread, const Impl::PerTeamValue& per_team) const {
+    return TeamPolicy(internal_policy::set_scratch_size(level, per_team, per_thread));
+  };
+
+};
 
 } // namespace Kokkos
 
