@@ -133,7 +133,11 @@
 // still identifies as 7.0
 #error "Cuda version 7.5 or greater required for host-to-device Lambda support"
 #endif
+#if ( CUDA_VERSION < 8000 )
 #define KOKKOS_LAMBDA [=]__device__
+#else
+#define KOKKOS_LAMBDA [=]__host__ __device__
+#endif
 #define KOKKOS_HAVE_CXX11_DISPATCH_LAMBDA 1
 #endif
 #endif /* #if defined( KOKKOS_HAVE_CUDA ) && defined( __CUDACC__ ) */
@@ -373,6 +377,12 @@
 #define KOKKOS_FUNCTION /**/
 #endif
 
+//----------------------------------------------------------------------------
+/** Define Macro for alignment: */
+#if ! defined(KOKKOS_ALIGN_16)
+#define KOKKOS_ALIGN_16 __attribute__((aligned(16)))
+#endif
+
 //----------------------------------------------------------------------------
 /** Determine the default execution space for parallel dispatch.
  *  There is zero or one default execution space specified.
@@ -422,5 +432,42 @@
 #define KOKKOS_POSIX_MEMALIGN_AVAILABLE 1
 #endif
 #endif
+
+//----------------------------------------------------------------------------
+//----------------------------------------------------------------------------
+
+/**Enable Profiling by default**/
+
+#ifndef KOKKOS_ENABLE_PROFILING
+#define KOKKOS_ENABLE_PROFILING 1
+#endif
+
+//----------------------------------------------------------------------------
+//----------------------------------------------------------------------------
+/* Transitional macro to change between old and new View,
+ * default to use new View.
+ */
+
+#if ! defined( KOKKOS_USING_EXP_VIEW )
+#if defined( KOKKOS_USING_DEPRECATED_VIEW )
+#define KOKKOS_USING_EXP_VIEW 0
+#else
+#define KOKKOS_USING_EXP_VIEW 1
+#endif
+#endif
+
+#if KOKKOS_USING_EXP_VIEW
+#if ! defined( KOKKOS_USING_EXPERIMENTAL_VIEW )
+#define KOKKOS_USING_EXPERIMENTAL_VIEW
+#endif
+#else /* ! KOKKOS_USING_EXP_VIEW */
+#if defined( KOKKOS_USING_EXPERIMENTAL_VIEW )
+#error "KOKKOS_USING_EXP_VIEW and KOKKOS_USING_EXPERIMENAL_VIEW are both defined and are incompatible"
+#endif
+#endif
+
+//----------------------------------------------------------------------------
+//----------------------------------------------------------------------------
+
 #endif /* #ifndef KOKKOS_MACROS_HPP */
 
