@@ -48,8 +48,6 @@
 
 namespace Kokkos { namespace Impl {
 
-class AllocationTracker;
-
 //The following tag is used to prevent an implicit call of the constructor when trying
 //to assign a literal 0 int ( = 0 );
 struct AtomicViewConstTag {};
@@ -427,7 +425,9 @@ struct Kokkos_Atomic_is_only_allowed_with_32bit_and_64bit_scalars<8> {
   typedef int64_t type;
 };
 
-#if ! defined( KOKKOS_USING_EXPERIMENTAL_VIEW )
+#if ! KOKKOS_USING_EXP_VIEW
+
+class AllocationTracker;
 
 // Must be non-const, atomic access trait, and 32 or 64 bit type for true atomics.
 template<class ViewTraits>
@@ -459,7 +459,7 @@ public:
   }
 };
 
-#endif /* #if ! defined( KOKKOS_USING_EXPERIMENTAL_VIEW ) */
+#endif /* #if ! KOKKOS_USING_EXP_VIEW */
 
 }} // namespace Kokkos::Impl
 
