@@ -44,6 +44,7 @@
 #ifndef KOKKOS_VIEWSUPPORT_HPP
 #define KOKKOS_VIEWSUPPORT_HPP
 
+#include <algorithm>
 #include <Kokkos_ExecPolicy.hpp>
 #include <impl/Kokkos_Shape.hpp>
 
@@ -114,6 +115,7 @@ template< class ExecSpace , class Type , bool Initialize >
 struct ViewDefaultConstruct
 { ViewDefaultConstruct( Type * , size_t ) {} };
 
+#if ! KOKKOS_USING_EXP_VIEW
 
 /** \brief  ViewDataHandle provides the type of the 'data handle' which the view
  *          uses to access data with the [] operator. It also provides
@@ -240,6 +242,8 @@ public:
     }
 };
 
+#endif /* #if ! KOKKOS_USING_EXP_VIEW */
+
 } // namespace Impl
 } // namespace Kokkos
 
@@ -391,8 +395,8 @@ struct ViewAllocateWithoutInitializing {
   const std::string label ;
 
   ViewAllocateWithoutInitializing() : label() {}
-  ViewAllocateWithoutInitializing( const std::string & arg_label ) : label( arg_label ) {}
-  ViewAllocateWithoutInitializing( const char * const  arg_label ) : label( arg_label ) {}
+  explicit ViewAllocateWithoutInitializing( const std::string & arg_label ) : label( arg_label ) {}
+  explicit ViewAllocateWithoutInitializing( const char * const  arg_label ) : label( arg_label ) {}
 };
 
 struct ViewAllocate {
