@@ -46,6 +46,10 @@
 #include <Kokkos_Macros.hpp>
 
 #if defined( KOKKOS_HAVE_PTHREAD )
+#ifdef KOKKOS_LAMBDA
+#undef KOKKOS_LAMBDA
+#endif
+#define KOKKOS_LAMBDA [=]
 
 #include <Kokkos_Core.hpp>
 
@@ -70,13 +74,18 @@
 #include <TestAggregate.hpp>
 #include <TestAggregateReduction.hpp>
 #include <TestCompilerMacros.hpp>
+#include <TestTaskPolicy.hpp>
+#include <TestMemoryPool.hpp>
+
+
 #include <TestCXX11.hpp>
 #include <TestCXX11Deduction.hpp>
 #include <TestTeamVector.hpp>
 #include <TestMemorySpaceTracking.hpp>
 #include <TestTemplateMetaFunctions.hpp>
 
-#include <TestTaskPolicy.hpp>
+
+#include <TestPolicyConstruction.hpp>
 
 namespace Test {
 
@@ -148,6 +157,11 @@ TEST_F( threads , impl_shared_alloc ) {
   test_shared_alloc< Kokkos::HostSpace , Kokkos::Threads >();
 }
 
+TEST_F( threads, policy_construction) {
+  TestRangePolicyConstruction< Kokkos::Threads >();
+  TestTeamPolicyConstruction< Kokkos::Threads >();
+}
+
 TEST_F( threads , impl_view_mapping ) {
   test_view_mapping< Kokkos::Threads >();
   test_view_mapping_subview< Kokkos::Threads >();
@@ -221,15 +235,21 @@ TEST_F( threads, view_aggregate ) {
 
 TEST_F( threads , range_tag )
 {
-  TestRange< Kokkos::Threads >::test_for(1000);
-  TestRange< Kokkos::Threads >::test_reduce(1000);
-  TestRange< Kokkos::Threads >::test_scan(1000);
+  TestRange< Kokkos::Threads , Kokkos::Schedule<Kokkos::Static> >::test_for(1000);
+  TestRange< Kokkos::Threads , Kokkos::Schedule<Kokkos::Static> >::test_reduce(1000);
+  TestRange< Kokkos::Threads , Kokkos::Schedule<Kokkos::Static> >::test_scan(1000);
+  TestRange< Kokkos::Threads , Kokkos::Schedule<Kokkos::Dynamic> >::test_for(1001);
+  TestRange< Kokkos::Threads , Kokkos::Schedule<Kokkos::Dynamic> >::test_reduce(1001);
+  TestRange< Kokkos::Threads , Kokkos::Schedule<Kokkos::Dynamic> >::test_scan(1001);
+  TestRange< Kokkos::Threads , Kokkos::Schedule<Kokkos::Dynamic> >::test_dynamic_policy(1000);
 }
 
 TEST_F( threads , team_tag )
 {
-  TestTeamPolicy< Kokkos::Threads >::test_for(1000);
-  TestTeamPolicy< Kokkos::Threads >::test_reduce(1000);
+  TestTeamPolicy< Kokkos::Threads , Kokkos::Schedule<Kokkos::Static> >::test_for(1000);
+  TestTeamPolicy< Kokkos::Threads , Kokkos::Schedule<Kokkos::Static> >::test_reduce(1000);
+  TestTeamPolicy< Kokkos::Threads , Kokkos::Schedule<Kokkos::Dynamic> >::test_for(1000);
+  TestTeamPolicy< Kokkos::Threads , Kokkos::Schedule<Kokkos::Dynamic> >::test_reduce(1000);
 }
 
 TEST_F( threads, long_reduce) {
@@ -241,11 +261,17 @@ TEST_F( threads, double_reduce) {
 }
 
 TEST_F( threads, team_long_reduce) {
-  TestReduceTeam< long ,   Kokkos::Threads >( 100000 );
+  TestReduceTeam< long ,   Kokkos::Threads , Kokkos::Schedule<Kokkos::Static> >( 3 );
+  TestReduceTeam< long ,   Kokkos::Threads , Kokkos::Schedule<Kokkos::Dynamic> >( 3 );
+  TestReduceTeam< long ,   Kokkos::Threads , Kokkos::Schedule<Kokkos::Static> >( 100000 );
+  TestReduceTeam< long ,   Kokkos::Threads , Kokkos::Schedule<Kokkos::Dynamic> >( 100000 );
 }
 
 TEST_F( threads, team_double_reduce) {
-  TestReduceTeam< double ,   Kokkos::Threads >( 100000 );
+  TestReduceTeam< double ,   Kokkos::Threads , Kokkos::Schedule<Kokkos::Static> >( 3 );
+  TestReduceTeam< double ,   Kokkos::Threads , Kokkos::Schedule<Kokkos::Dynamic> >( 3 );
+  TestReduceTeam< double ,   Kokkos::Threads , Kokkos::Schedule<Kokkos::Static> >( 100000 );
+  TestReduceTeam< double ,   Kokkos::Threads , Kokkos::Schedule<Kokkos::Dynamic> >( 100000 );
 }
 
 TEST_F( threads, long_reduce_dynamic ) {
@@ -261,12 +287,14 @@ TEST_F( threads, long_reduce_dynamic_view ) {
 }
 
 TEST_F( threads, team_shared_request) {
-  TestSharedTeam< Kokkos::Threads >();
+  TestSharedTeam< Kokkos::Threads , Kokkos::Schedule<Kokkos::Static> >();
+  TestSharedTeam< Kokkos::Threads , Kokkos::Schedule<Kokkos::Dynamic> >();
 }
 
-#if defined(KOKKOS_HAVE_CXX11_DISPATCH_LAMBDA) && !defined(KOKKOS_HAVE_CUDA)
+#if defined(KOKKOS_HAVE_CXX11_DISPATCH_LAMBDA) 
 TEST_F( threads, team_lambda_shared_request) {
-  TestLambdaSharedTeam< Kokkos::Threads >();
+  TestLambdaSharedTeam< Kokkos::HostSpace, Kokkos::Threads , Kokkos::Schedule<Kokkos::Static> >();
+  TestLambdaSharedTeam< Kokkos::HostSpace, Kokkos::Threads , Kokkos::Schedule<Kokkos::Dynamic> >();
 }
 #endif
 
@@ -345,14 +373,13 @@ TEST_F( threads , atomics )
   ASSERT_TRUE( ( TestAtomic::Loop<float,Kokkos::Threads>(100,2) ) );
   ASSERT_TRUE( ( TestAtomic::Loop<float,Kokkos::Threads>(100,3) ) );
 
-#if defined( KOKKOS_ENABLE_ASM )
   ASSERT_TRUE( ( TestAtomic::Loop<Kokkos::complex<double> ,Kokkos::Threads>(100,1) ) );
   ASSERT_TRUE( ( TestAtomic::Loop<Kokkos::complex<double> ,Kokkos::Threads>(100,2) ) );
   ASSERT_TRUE( ( TestAtomic::Loop<Kokkos::complex<double> ,Kokkos::Threads>(100,3) ) );
-#endif
 
   ASSERT_TRUE( ( TestAtomic::Loop<TestAtomic::SuperScalar<3>, Kokkos::Threads>(loop_count,1) ) );
   ASSERT_TRUE( ( TestAtomic::Loop<TestAtomic::SuperScalar<3>, Kokkos::Threads>(loop_count,2) ) );
+  ASSERT_TRUE( ( TestAtomic::Loop<TestAtomic::SuperScalar<3>, Kokkos::Threads>(loop_count,3) ) );
 }
 
 //----------------------------------------------------------------------------
@@ -384,8 +411,10 @@ TEST_F( threads , scan )
 
 TEST_F( threads , team_scan )
 {
-  TestScanTeam< Kokkos::Threads >( 10 );
-  TestScanTeam< Kokkos::Threads >( 10000 );
+  TestScanTeam< Kokkos::Threads , Kokkos::Schedule<Kokkos::Static> >( 10 );
+  TestScanTeam< Kokkos::Threads , Kokkos::Schedule<Kokkos::Dynamic> >( 10 );
+  TestScanTeam< Kokkos::Threads , Kokkos::Schedule<Kokkos::Static> >( 10000 );
+  TestScanTeam< Kokkos::Threads , Kokkos::Schedule<Kokkos::Dynamic> >( 10000 );
 }
 
 //----------------------------------------------------------------------------
@@ -400,6 +429,14 @@ TEST_F( threads , memory_space )
   TestMemorySpace< Kokkos::Threads >();
 }
 
+TEST_F( threads , memory_pool )
+{
+  bool val = TestMemoryPool::test_mempool< Kokkos::Threads >( 128, 128000000 );
+  ASSERT_TRUE( val );
+
+  TestMemoryPool::test_mempool2< Kokkos::Threads >( 64, 4, 1000000, 2000000 );
+}
+
 //----------------------------------------------------------------------------
 
 TEST_F( threads , template_meta_functions )
@@ -444,8 +481,15 @@ TEST_F( threads , team_vector )
 TEST_F( threads , task_policy )
 {
   TestTaskPolicy::test_task_dep< Kokkos::Threads >( 10 );
-  for ( long i = 0 ; i < 25 ; ++i ) TestTaskPolicy::test_fib< Kokkos::Threads >(i);
-  for ( long i = 0 ; i < 35 ; ++i ) TestTaskPolicy::test_fib2< Kokkos::Threads >(i);
+
+  for ( long i = 0 ; i < 25 ; ++i ) {
+//    printf( "test_fib():  %2ld\n", i );
+    TestTaskPolicy::test_fib< Kokkos::Threads >(i);
+  }
+  for ( long i = 0 ; i < 35 ; ++i ) {
+//    printf( "test_fib2(): %2ld\n", i );
+    TestTaskPolicy::test_fib2< Kokkos::Threads >(i);
+  }
 }
 
 TEST_F( threads , task_team )
@@ -453,6 +497,12 @@ TEST_F( threads , task_team )
   TestTaskPolicy::test_task_team< Kokkos::Threads >(1000);
 }
 
+TEST_F( threads , task_latch )
+{
+  TestTaskPolicy::test_latch< Kokkos::Threads >(10);
+  TestTaskPolicy::test_latch< Kokkos::Threads >(1000);
+}
+
 } // namespace Test
 
 #endif /* #if defined( KOKKOS_HAVE_PTHREAD ) */
